# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'DAQ_Navigation_Visu_GUI.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(878, 848)
        MainWindow.setBaseSize(QtCore.QSize(1000, 800))
        MainWindow.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.centralwidget.sizePolicy().hasHeightForWidth())
        self.centralwidget.setSizePolicy(sizePolicy)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.splitter_7 = QtWidgets.QSplitter(self.centralwidget)
        self.splitter_7.setOrientation(QtCore.Qt.Horizontal)
        self.splitter_7.setObjectName("splitter_7")
        self.tabWidget_2 = QtWidgets.QTabWidget(self.splitter_7)
        self.tabWidget_2.setObjectName("tabWidget_2")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.gridLayout_9 = QtWidgets.QGridLayout(self.tab_2)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.plotting_layout = QtWidgets.QVBoxLayout()
        self.plotting_layout.setObjectName("plotting_layout")
        self.gridLayout_9.addLayout(self.plotting_layout, 0, 0, 1, 1)
        self.tabWidget_2.addTab(self.tab_2, "")
        self.tab_3 = QtWidgets.QWidget()
        self.tab_3.setObjectName("tab_3")
        self.gridLayout_17 = QtWidgets.QGridLayout(self.tab_3)
        self.gridLayout_17.setObjectName("gridLayout_17")
        self.splitter_5 = QtWidgets.QSplitter(self.tab_3)
        self.splitter_5.setOrientation(QtCore.Qt.Vertical)
        self.splitter_5.setObjectName("splitter_5")
        self.splitter_6 = QtWidgets.QSplitter(self.splitter_5)
        self.splitter_6.setOrientation(QtCore.Qt.Horizontal)
        self.splitter_6.setObjectName("splitter_6")
        self.verticalLayoutWidget_3 = QtWidgets.QWidget(self.splitter_6)
        self.verticalLayoutWidget_3.setObjectName("verticalLayoutWidget_3")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.verticalLayoutWidget_3)
        self.verticalLayout_8.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.Tree = QtWidgets.QTreeWidget(self.verticalLayoutWidget_3)
        self.Tree.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.Tree.setObjectName("Tree")
        self.Tree.headerItem().setText(0, "1")
        self.verticalLayout_8.addWidget(self.Tree)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.Open_Tree = QtWidgets.QPushButton(self.verticalLayoutWidget_3)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/Labview_icons/Icon_Library/tree.png"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.Open_Tree.setIcon(icon)
        self.Open_Tree.setObjectName("Open_Tree")
        self.horizontalLayout_2.addWidget(self.Open_Tree)
        self.Open_Tree_Selected = QtWidgets.QPushButton(self.verticalLayoutWidget_3)
        self.Open_Tree_Selected.setIcon(icon)
        self.Open_Tree_Selected.setObjectName("Open_Tree_Selected")
        self.horizontalLayout_2.addWidget(self.Open_Tree_Selected)
        self.Close_Tree = QtWidgets.QPushButton(self.verticalLayoutWidget_3)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/Labview_icons/Icon_Library/CollapseAll.png"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.Close_Tree.setIcon(icon1)
        self.Close_Tree.setObjectName("Close_Tree")
        self.horizontalLayout_2.addWidget(self.Close_Tree)
        self.verticalLayout_8.addLayout(self.horizontalLayout_2)
        self.verticalLayoutWidget_4 = QtWidgets.QWidget(self.splitter_6)
        self.verticalLayoutWidget_4.setObjectName("verticalLayoutWidget_4")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.verticalLayoutWidget_4)
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.Data1D_graph = PlotWidget(self.splitter_5)
        self.Data1D_graph.setObjectName("Data1D_graph")
        self.gridLayout_17.addWidget(self.splitter_5, 0, 0, 1, 1)
        self.tabWidget_2.addTab(self.tab_3, "")
        self.splitter_3 = QtWidgets.QSplitter(self.splitter_7)
        self.splitter_3.setMinimumSize(QtCore.QSize(300, 0))
        self.splitter_3.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.splitter_3.setOrientation(QtCore.Qt.Vertical)
        self.splitter_3.setObjectName("splitter_3")
        self.tabWidget = QtWidgets.QTabWidget(self.splitter_3)
        self.tabWidget.setBaseSize(QtCore.QSize(0, 0))
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.tab)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.splitter_4 = QtWidgets.QSplitter(self.tab)
        self.splitter_4.setOrientation(QtCore.Qt.Vertical)
        self.splitter_4.setObjectName("splitter_4")
        self.layoutWidget = QtWidgets.QWidget(self.splitter_4)
        self.layoutWidget.setObjectName("layoutWidget")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.layoutWidget)
        self.gridLayout_7.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.save_process_pb = QtWidgets.QPushButton(self.layoutWidget)
        self.save_process_pb.setObjectName("save_process_pb")
        self.gridLayout_7.addWidget(self.save_process_pb, 7, 2, 1, 1)
        self.process_data_pb = QtWidgets.QPushButton(self.layoutWidget)
        self.process_data_pb.setObjectName("process_data_pb")
        self.gridLayout_7.addWidget(self.process_data_pb, 7, 0, 1, 2)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.result_edit_blue = QtWidgets.QLineEdit(self.layoutWidget)
        self.result_edit_blue.setEnabled(True)
        self.result_edit_blue.setReadOnly(True)
        self.result_edit_blue.setObjectName("result_edit_blue")
        self.horizontalLayout_4.addWidget(self.result_edit_blue)
        self.result_edit_green = QtWidgets.QLineEdit(self.layoutWidget)
        self.result_edit_green.setEnabled(True)
        self.result_edit_green.setReadOnly(True)
        self.result_edit_green.setObjectName("result_edit_green")
        self.horizontalLayout_4.addWidget(self.result_edit_green)
        self.result_edit_red = QtWidgets.QLineEdit(self.layoutWidget)
        self.result_edit_red.setEnabled(True)
        self.result_edit_red.setReadOnly(True)
        self.result_edit_red.setObjectName("result_edit_red")
        self.horizontalLayout_4.addWidget(self.result_edit_red)
        self.gridLayout_7.addLayout(self.horizontalLayout_4, 6, 0, 1, 3)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.result_blue_label = QtWidgets.QPushButton(self.layoutWidget)
        self.result_blue_label.setEnabled(True)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/Labview_icons/Icon_Library/b_icon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.result_blue_label.setIcon(icon2)
        self.result_blue_label.setFlat(True)
        self.result_blue_label.setObjectName("result_blue_label")
        self.horizontalLayout_5.addWidget(self.result_blue_label)
        self.result_green_label = QtWidgets.QPushButton(self.layoutWidget)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/Labview_icons/Icon_Library/g_icon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.result_green_label.setIcon(icon3)
        self.result_green_label.setFlat(True)
        self.result_green_label.setObjectName("result_green_label")
        self.horizontalLayout_5.addWidget(self.result_green_label)
        self.result_red_label = QtWidgets.QPushButton(self.layoutWidget)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/Labview_icons/Icon_Library/r_icon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.result_red_label.setIcon(icon4)
        self.result_red_label.setFlat(True)
        self.result_red_label.setObjectName("result_red_label")
        self.horizontalLayout_5.addWidget(self.result_red_label)
        self.gridLayout_7.addLayout(self.horizontalLayout_5, 5, 0, 1, 3)
        self.colors_combo = QtWidgets.QComboBox(self.layoutWidget)
        self.colors_combo.setObjectName("colors_combo")
        self.gridLayout_7.addWidget(self.colors_combo, 0, 0, 1, 1)
        self.merge_spectra_cb = QtWidgets.QCheckBox(self.layoutWidget)
        self.merge_spectra_cb.setObjectName("merge_spectra_cb")
        self.gridLayout_7.addWidget(self.merge_spectra_cb, 0, 1, 1, 2)
        self.label_8 = QtWidgets.QLabel(self.layoutWidget)
        self.label_8.setObjectName("label_8")
        self.gridLayout_7.addWidget(self.label_8, 9, 0, 1, 3)
        self.comments_edit = QtWidgets.QTextEdit(self.layoutWidget)
        self.comments_edit.setMaximumSize(QtCore.QSize(16777215, 50))
        self.comments_edit.setObjectName("comments_edit")
        self.gridLayout_7.addWidget(self.comments_edit, 10, 0, 1, 3)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_7.addItem(spacerItem, 11, 0, 1, 3)
        self.toolBox_Analysis = QtWidgets.QToolBox(self.splitter_4)
        self.toolBox_Analysis.setFrameShape(QtWidgets.QFrame.Panel)
        self.toolBox_Analysis.setFrameShadow(QtWidgets.QFrame.Raised)
        self.toolBox_Analysis.setObjectName("toolBox_Analysis")
        self.page_measurement = QtWidgets.QWidget()
        self.page_measurement.setGeometry(QtCore.QRect(0, 0, 293, 214))
        self.page_measurement.setObjectName("page_measurement")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.page_measurement)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.label_7 = QtWidgets.QLabel(self.page_measurement)
        self.label_7.setAlignment(QtCore.Qt.AlignCenter)
        self.label_7.setObjectName("label_7")
        self.gridLayout.addWidget(self.label_7, 5, 0, 1, 1)
        self.operation_edit = QtWidgets.QTextEdit(self.page_measurement)
        self.operation_edit.setMaximumSize(QtCore.QSize(16777215, 55))
        self.operation_edit.setObjectName("operation_edit")
        self.gridLayout.addWidget(self.operation_edit, 6, 1, 1, 1)
        self.label_20 = QtWidgets.QLabel(self.page_measurement)
        self.label_20.setObjectName("label_20")
        self.gridLayout.addWidget(self.label_20, 6, 0, 1, 1)
        self.measure_subtype_combo = QtWidgets.QComboBox(self.page_measurement)
        self.measure_subtype_combo.setObjectName("measure_subtype_combo")
        self.measure_subtype_combo.addItem("")
        self.measure_subtype_combo.addItem("")
        self.measure_subtype_combo.addItem("")
        self.gridLayout.addWidget(self.measure_subtype_combo, 2, 1, 1, 1)
        self.variables_edit = QtWidgets.QLineEdit(self.page_measurement)
        self.variables_edit.setText("")
        self.variables_edit.setReadOnly(True)
        self.variables_edit.setObjectName("variables_edit")
        self.gridLayout.addWidget(self.variables_edit, 3, 1, 1, 1)
        self.label_9 = QtWidgets.QLabel(self.page_measurement)
        self.label_9.setAlignment(QtCore.Qt.AlignCenter)
        self.label_9.setObjectName("label_9")
        self.gridLayout.addWidget(self.label_9, 0, 0, 1, 1)
        self.label_11 = QtWidgets.QLabel(self.page_measurement)
        self.label_11.setAlignment(QtCore.Qt.AlignCenter)
        self.label_11.setObjectName("label_11")
        self.gridLayout.addWidget(self.label_11, 2, 0, 1, 1)
        self.measure_type_combo = QtWidgets.QComboBox(self.page_measurement)
        self.measure_type_combo.setObjectName("measure_type_combo")
        self.measure_type_combo.addItem("")
        self.measure_type_combo.addItem("")
        self.measure_type_combo.addItem("")
        self.measure_type_combo.addItem("")
        self.measure_type_combo.addItem("")
        self.measure_type_combo.addItem("")
        self.gridLayout.addWidget(self.measure_type_combo, 0, 1, 1, 1)
        self.formula_edit = QtWidgets.QPlainTextEdit(self.page_measurement)
        self.formula_edit.setMaximumSize(QtCore.QSize(16777215, 50))
        self.formula_edit.setPlainText("")
        self.formula_edit.setTextInteractionFlags(QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.formula_edit.setObjectName("formula_edit")
        self.gridLayout.addWidget(self.formula_edit, 5, 1, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.page_measurement)
        self.label_2.setAlignment(QtCore.Qt.AlignCenter)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 3, 0, 1, 1)
        self.do_operation_cb = QtWidgets.QCheckBox(self.page_measurement)
        self.do_operation_cb.setObjectName("do_operation_cb")
        self.gridLayout.addWidget(self.do_operation_cb, 7, 0, 1, 1)
        self.gridLayout_2.addLayout(self.gridLayout, 0, 0, 1, 1)
        self.toolBox_Analysis.addItem(self.page_measurement, "")
        self.page_bkg = QtWidgets.QWidget()
        self.page_bkg.setGeometry(QtCore.QRect(0, 0, 480, 115))
        self.page_bkg.setObjectName("page_bkg")
        self.gridLayout_14 = QtWidgets.QGridLayout(self.page_bkg)
        self.gridLayout_14.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_14.setObjectName("gridLayout_14")
        self.gridLayout_13 = QtWidgets.QGridLayout()
        self.gridLayout_13.setObjectName("gridLayout_13")
        self.apply_bkg_check = QtWidgets.QCheckBox(self.page_bkg)
        self.apply_bkg_check.setObjectName("apply_bkg_check")
        self.gridLayout_13.addWidget(self.apply_bkg_check, 1, 0, 1, 1)
        self.set_whole_as_bkg_cb = QtWidgets.QCheckBox(self.page_bkg)
        self.set_whole_as_bkg_cb.setObjectName("set_whole_as_bkg_cb")
        self.gridLayout_13.addWidget(self.set_whole_as_bkg_cb, 0, 0, 1, 2)
        self.bkg_type = QtWidgets.QComboBox(self.page_bkg)
        self.bkg_type.setObjectName("bkg_type")
        self.bkg_type.addItem("")
        self.bkg_type.addItem("")
        self.gridLayout_13.addWidget(self.bkg_type, 1, 1, 1, 1)
        self.background_edit = QtWidgets.QLineEdit(self.page_bkg)
        self.background_edit.setReadOnly(True)
        self.background_edit.setObjectName("background_edit")
        self.gridLayout_13.addWidget(self.background_edit, 2, 0, 1, 3)
        self.do_zero_cb = QtWidgets.QCheckBox(self.page_bkg)
        self.do_zero_cb.setObjectName("do_zero_cb")
        self.gridLayout_13.addWidget(self.do_zero_cb, 1, 2, 1, 1)
        self.set_as_bkg_pb = QtWidgets.QPushButton(self.page_bkg)
        self.set_as_bkg_pb.setObjectName("set_as_bkg_pb")
        self.gridLayout_13.addWidget(self.set_as_bkg_pb, 0, 2, 1, 1)
        self.gridLayout_14.addLayout(self.gridLayout_13, 0, 0, 1, 1)
        self.toolBox_Analysis.addItem(self.page_bkg, "")
        self.page_spikes = QtWidgets.QWidget()
        self.page_spikes.setGeometry(QtCore.QRect(0, 0, 202, 123))
        self.page_spikes.setObjectName("page_spikes")
        self.gridLayout_16 = QtWidgets.QGridLayout(self.page_spikes)
        self.gridLayout_16.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_16.setHorizontalSpacing(6)
        self.gridLayout_16.setObjectName("gridLayout_16")
        self.gridLayout_15 = QtWidgets.QGridLayout()
        self.gridLayout_15.setObjectName("gridLayout_15")
        self.show_despiked_cb = QtWidgets.QCheckBox(self.page_spikes)
        self.show_despiked_cb.setObjectName("show_despiked_cb")
        self.gridLayout_15.addWidget(self.show_despiked_cb, 0, 1, 1, 1)
        self.threshold_sb = QtWidgets.QDoubleSpinBox(self.page_spikes)
        self.threshold_sb.setMinimum(-100000.0)
        self.threshold_sb.setMaximum(100000.0)
        self.threshold_sb.setSingleStep(0.01)
        self.threshold_sb.setProperty("value", 0.8)
        self.threshold_sb.setObjectName("threshold_sb")
        self.gridLayout_15.addWidget(self.threshold_sb, 1, 1, 1, 1)
        self.label_12 = QtWidgets.QLabel(self.page_spikes)
        self.label_12.setObjectName("label_12")
        self.gridLayout_15.addWidget(self.label_12, 1, 0, 1, 1)
        self.remove_spikes_cb = QtWidgets.QCheckBox(self.page_spikes)
        self.remove_spikes_cb.setObjectName("remove_spikes_cb")
        self.gridLayout_15.addWidget(self.remove_spikes_cb, 0, 0, 1, 1)
        self.label_16 = QtWidgets.QLabel(self.page_spikes)
        self.label_16.setObjectName("label_16")
        self.gridLayout_15.addWidget(self.label_16, 2, 0, 1, 1)
        self.despiked_width_spikes_sb = QtWidgets.QSpinBox(self.page_spikes)
        self.despiked_width_spikes_sb.setMinimum(1)
        self.despiked_width_spikes_sb.setMaximum(1000)
        self.despiked_width_spikes_sb.setProperty("value", 2)
        self.despiked_width_spikes_sb.setObjectName("despiked_width_spikes_sb")
        self.gridLayout_15.addWidget(self.despiked_width_spikes_sb, 2, 1, 1, 1)
        self.despiked_width_average_sb = QtWidgets.QSpinBox(self.page_spikes)
        self.despiked_width_average_sb.setMinimum(1)
        self.despiked_width_average_sb.setMaximum(1000)
        self.despiked_width_average_sb.setProperty("value", 6)
        self.despiked_width_average_sb.setObjectName("despiked_width_average_sb")
        self.gridLayout_15.addWidget(self.despiked_width_average_sb, 4, 1, 1, 1)
        self.label_17 = QtWidgets.QLabel(self.page_spikes)
        self.label_17.setObjectName("label_17")
        self.gridLayout_15.addWidget(self.label_17, 4, 0, 1, 1)
        self.label_18 = QtWidgets.QLabel(self.page_spikes)
        self.label_18.setObjectName("label_18")
        self.gridLayout_15.addWidget(self.label_18, 5, 0, 1, 1)
        self.despiked_Nloop_sb = QtWidgets.QSpinBox(self.page_spikes)
        self.despiked_Nloop_sb.setMinimum(1)
        self.despiked_Nloop_sb.setMaximum(1000)
        self.despiked_Nloop_sb.setProperty("value", 2)
        self.despiked_Nloop_sb.setObjectName("despiked_Nloop_sb")
        self.gridLayout_15.addWidget(self.despiked_Nloop_sb, 5, 1, 1, 1)
        self.gridLayout_16.addLayout(self.gridLayout_15, 0, 0, 1, 1)
        self.toolBox_Analysis.addItem(self.page_spikes, "")
        self.gridLayout_8.addWidget(self.splitter_4, 0, 0, 1, 1)
        self.tabWidget.addTab(self.tab, "")
        self.tab_TreeProperties = QtWidgets.QWidget()
        self.tab_TreeProperties.setObjectName("tab_TreeProperties")
        self.gridLayout_10 = QtWidgets.QGridLayout(self.tab_TreeProperties)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.table_properties = QtWidgets.QTableWidget(self.tab_TreeProperties)
        self.table_properties.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustToContents)
        self.table_properties.setAlternatingRowColors(True)
        self.table_properties.setWordWrap(False)
        self.table_properties.setObjectName("table_properties")
        self.table_properties.setColumnCount(0)
        self.table_properties.setRowCount(0)
        self.gridLayout_10.addWidget(self.table_properties, 0, 0, 1, 1)
        self.tabWidget.addTab(self.tab_TreeProperties, "")
        self.tab_plotting = QtWidgets.QWidget()
        self.tab_plotting.setObjectName("tab_plotting")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.tab_plotting)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.gridLayout_4 = QtWidgets.QGridLayout()
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.label_15 = QtWidgets.QLabel(self.tab_plotting)
        self.label_15.setObjectName("label_15")
        self.gridLayout_4.addWidget(self.label_15, 10, 0, 1, 1)
        self.label_14 = QtWidgets.QLabel(self.tab_plotting)
        self.label_14.setObjectName("label_14")
        self.gridLayout_4.addWidget(self.label_14, 9, 0, 1, 1)
        self.label_10 = QtWidgets.QLabel(self.tab_plotting)
        self.label_10.setObjectName("label_10")
        self.gridLayout_4.addWidget(self.label_10, 5, 0, 2, 1)
        self.aspect_combo = QtWidgets.QComboBox(self.tab_plotting)
        self.aspect_combo.setObjectName("aspect_combo")
        self.aspect_combo.addItem("")
        self.aspect_combo.addItem("")
        self.gridLayout_4.addWidget(self.aspect_combo, 5, 1, 2, 1)
        self.plot_2D_pb = QtWidgets.QPushButton(self.tab_plotting)
        self.plot_2D_pb.setObjectName("plot_2D_pb")
        self.gridLayout_4.addWidget(self.plot_2D_pb, 0, 0, 2, 1)
        self.plot_1D_pb = QtWidgets.QPushButton(self.tab_plotting)
        self.plot_1D_pb.setObjectName("plot_1D_pb")
        self.gridLayout_4.addWidget(self.plot_1D_pb, 0, 1, 2, 1)
        self.cmap_combo = QtWidgets.QComboBox(self.tab_plotting)
        self.cmap_combo.setObjectName("cmap_combo")
        self.gridLayout_4.addWidget(self.cmap_combo, 3, 1, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem1, 0, 2, 13, 2)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem2, 13, 0, 1, 4)
        self.label = QtWidgets.QLabel(self.tab_plotting)
        self.label.setObjectName("label")
        self.gridLayout_4.addWidget(self.label, 3, 0, 1, 1)
        self.cmap_graphicsView = QtWidgets.QGraphicsView(self.tab_plotting)
        self.cmap_graphicsView.setMaximumSize(QtCore.QSize(2000, 25))
        self.cmap_graphicsView.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.cmap_graphicsView.setObjectName("cmap_graphicsView")
        self.gridLayout_4.addWidget(self.cmap_graphicsView, 4, 0, 1, 2)
        self.colorbar_cb = QtWidgets.QCheckBox(self.tab_plotting)
        self.colorbar_cb.setObjectName("colorbar_cb")
        self.gridLayout_4.addWidget(self.colorbar_cb, 7, 0, 1, 1)
        self.update_plot_pb = QtWidgets.QPushButton(self.tab_plotting)
        self.update_plot_pb.setObjectName("update_plot_pb")
        self.gridLayout_4.addWidget(self.update_plot_pb, 7, 1, 1, 1)
        self.save_plotting_pb = QtWidgets.QPushButton(self.tab_plotting)
        self.save_plotting_pb.setObjectName("save_plotting_pb")
        self.gridLayout_4.addWidget(self.save_plotting_pb, 12, 0, 1, 2)
        self.label_13 = QtWidgets.QLabel(self.tab_plotting)
        self.label_13.setObjectName("label_13")
        self.gridLayout_4.addWidget(self.label_13, 8, 0, 1, 1)
        self.use_labels_cb = QtWidgets.QCheckBox(self.tab_plotting)
        self.use_labels_cb.setChecked(True)
        self.use_labels_cb.setObjectName("use_labels_cb")
        self.gridLayout_4.addWidget(self.use_labels_cb, 11, 0, 1, 1)
        self.xlabel_edit = QtWidgets.QLineEdit(self.tab_plotting)
        self.xlabel_edit.setObjectName("xlabel_edit")
        self.gridLayout_4.addWidget(self.xlabel_edit, 8, 1, 1, 1)
        self.ylabel_edit = QtWidgets.QLineEdit(self.tab_plotting)
        self.ylabel_edit.setObjectName("ylabel_edit")
        self.gridLayout_4.addWidget(self.ylabel_edit, 9, 1, 1, 1)
        self.title_edit = QtWidgets.QLineEdit(self.tab_plotting)
        self.title_edit.setObjectName("title_edit")
        self.gridLayout_4.addWidget(self.title_edit, 10, 1, 1, 1)
        self.gridLayout_6.addLayout(self.gridLayout_4, 0, 0, 1, 1)
        self.tabWidget.addTab(self.tab_plotting, "")
        self.toolBox = QtWidgets.QToolBox(self.splitter_3)
        self.toolBox.setObjectName("toolBox")
        self.page = QtWidgets.QWidget()
        self.page.setGeometry(QtCore.QRect(0, 0, 506, 223))
        self.page.setObjectName("page")
        self.gridLayout_12 = QtWidgets.QGridLayout(self.page)
        self.gridLayout_12.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_12.setObjectName("gridLayout_12")
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setHorizontalSpacing(6)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.label_5 = QtWidgets.QLabel(self.page)
        self.label_5.setObjectName("label_5")
        self.gridLayout_3.addWidget(self.label_5, 2, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.page)
        self.label_4.setObjectName("label_4")
        self.gridLayout_3.addWidget(self.label_4, 1, 0, 1, 1)
        self.Current_Carto_edit = QtWidgets.QLineEdit(self.page)
        self.Current_Carto_edit.setEnabled(False)
        self.Current_Carto_edit.setObjectName("Current_Carto_edit")
        self.gridLayout_3.addWidget(self.Current_Carto_edit, 1, 1, 1, 1)
        self.Current_Group_edit = QtWidgets.QLineEdit(self.page)
        self.Current_Group_edit.setEnabled(False)
        self.Current_Group_edit.setObjectName("Current_Group_edit")
        self.gridLayout_3.addWidget(self.Current_Group_edit, 2, 1, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.page)
        self.label_6.setObjectName("label_6")
        self.gridLayout_3.addWidget(self.label_6, 4, 0, 1, 1)
        self.Current_Channel_edit = QtWidgets.QPlainTextEdit(self.page)
        self.Current_Channel_edit.setEnabled(False)
        self.Current_Channel_edit.setMaximumSize(QtCore.QSize(16777215, 50))
        self.Current_Channel_edit.setObjectName("Current_Channel_edit")
        self.gridLayout_3.addWidget(self.Current_Channel_edit, 4, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.page)
        self.label_3.setObjectName("label_3")
        self.gridLayout_3.addWidget(self.label_3, 6, 0, 2, 1)
        self.Filenamepath = QtWidgets.QPlainTextEdit(self.page)
        self.Filenamepath.setMinimumSize(QtCore.QSize(0, 0))
        self.Filenamepath.setMaximumSize(QtCore.QSize(16777215, 50))
        self.Filenamepath.setObjectName("Filenamepath")
        self.gridLayout_3.addWidget(self.Filenamepath, 6, 1, 2, 1)
        self.gridLayout_12.addLayout(self.gridLayout_3, 0, 0, 1, 1)
        self.toolBox.addItem(self.page, "")
        self.gridLayout_5.addWidget(self.splitter_7, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 878, 21))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuLoad_Cartography_Labview = QtWidgets.QMenu(self.menuFile)
        self.menuLoad_Cartography_Labview.setObjectName("menuLoad_Cartography_Labview")
        self.menuAdd_Cartography = QtWidgets.QMenu(self.menuFile)
        self.menuAdd_Cartography.setObjectName("menuAdd_Cartography")
        self.menuExport = QtWidgets.QMenu(self.menuFile)
        self.menuExport.setObjectName("menuExport")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionLoad_TDMS = QtWidgets.QAction(MainWindow)
        self.actionLoad_TDMS.setObjectName("actionLoad_TDMS")
        self.actionLoad_Cartography_Labspec = QtWidgets.QAction(MainWindow)
        self.actionLoad_Cartography_Labspec.setObjectName("actionLoad_Cartography_Labspec")
        self.actionLoad_Cartography_separated_files = QtWidgets.QAction(MainWindow)
        self.actionLoad_Cartography_separated_files.setObjectName("actionLoad_Cartography_separated_files")
        self.actionLoad_1D_Trace = QtWidgets.QAction(MainWindow)
        self.actionLoad_1D_Trace.setObjectName("actionLoad_1D_Trace")
        self.actionLoad_Cartography_AFM = QtWidgets.QAction(MainWindow)
        self.actionLoad_Cartography_AFM.setObjectName("actionLoad_Cartography_AFM")
        self.actionDelete_Tree_Element = QtWidgets.QAction(MainWindow)
        self.actionDelete_Tree_Element.setObjectName("actionDelete_Tree_Element")
        self.actionLabview = QtWidgets.QAction(MainWindow)
        self.actionLabview.setObjectName("actionLabview")
        self.actionSeparated_Files = QtWidgets.QAction(MainWindow)
        self.actionSeparated_Files.setObjectName("actionSeparated_Files")
        self.actionAFM = QtWidgets.QAction(MainWindow)
        self.actionAFM.setObjectName("actionAFM")
        self.actionLabview_add = QtWidgets.QAction(MainWindow)
        self.actionLabview_add.setObjectName("actionLabview_add")
        self.actionLabspec_add = QtWidgets.QAction(MainWindow)
        self.actionLabspec_add.setObjectName("actionLabspec_add")
        self.actionSeparated_Files_add = QtWidgets.QAction(MainWindow)
        self.actionSeparated_Files_add.setObjectName("actionSeparated_Files_add")
        self.actionAFM_add = QtWidgets.QAction(MainWindow)
        self.actionAFM_add.setObjectName("actionAFM_add")
        self.action2D_image_as_txt = QtWidgets.QAction(MainWindow)
        self.action2D_image_as_txt.setObjectName("action2D_image_as_txt")
        self.actionQuit = QtWidgets.QAction(MainWindow)
        self.actionQuit.setObjectName("actionQuit")
        self.actionLabspec_dual_mode = QtWidgets.QAction(MainWindow)
        self.actionLabspec_dual_mode.setObjectName("actionLabspec_dual_mode")
        self.actionLabspec_Dual_Mode = QtWidgets.QAction(MainWindow)
        self.actionLabspec_Dual_Mode.setObjectName("actionLabspec_Dual_Mode")
        self.actionLabspec = QtWidgets.QAction(MainWindow)
        self.actionLabspec.setObjectName("actionLabspec")
        self.actionLabspec_Dual_Mode_add = QtWidgets.QAction(MainWindow)
        self.actionLabspec_Dual_Mode_add.setObjectName("actionLabspec_Dual_Mode_add")
        self.menuLoad_Cartography_Labview.addAction(self.actionLabview)
        self.menuLoad_Cartography_Labview.addAction(self.actionLabspec)
        self.menuLoad_Cartography_Labview.addAction(self.actionLabspec_Dual_Mode)
        self.menuLoad_Cartography_Labview.addAction(self.actionSeparated_Files)
        self.menuLoad_Cartography_Labview.addAction(self.actionAFM)
        self.menuLoad_Cartography_Labview.addSeparator()
        self.menuAdd_Cartography.addAction(self.actionLabview_add)
        self.menuAdd_Cartography.addAction(self.actionLabspec_add)
        self.menuAdd_Cartography.addAction(self.actionLabspec_Dual_Mode_add)
        self.menuAdd_Cartography.addAction(self.actionSeparated_Files_add)
        self.menuAdd_Cartography.addAction(self.actionAFM_add)
        self.menuExport.addAction(self.action2D_image_as_txt)
        self.menuFile.addAction(self.actionLoad_TDMS)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.menuLoad_Cartography_Labview.menuAction())
        self.menuFile.addAction(self.menuAdd_Cartography.menuAction())
        self.menuFile.addAction(self.actionLoad_1D_Trace)
        self.menuFile.addAction(self.actionDelete_Tree_Element)
        self.menuFile.addAction(self.menuExport.menuAction())
        self.menuFile.addAction(self.actionQuit)
        self.menubar.addAction(self.menuFile.menuAction())

        self.retranslateUi(MainWindow)
        self.tabWidget_2.setCurrentIndex(1)
        self.tabWidget.setCurrentIndex(0)
        self.toolBox_Analysis.setCurrentIndex(1)
        self.toolBox.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.tabWidget_2.setTabText(self.tabWidget_2.indexOf(self.tab_2), _translate("MainWindow", "Plotting"))
        self.Open_Tree.setText(_translate("MainWindow", "Open Tree"))
        self.Open_Tree_Selected.setText(_translate("MainWindow", "Open Selected"))
        self.Close_Tree.setText(_translate("MainWindow", "Close Tree"))
        self.tabWidget_2.setTabText(self.tabWidget_2.indexOf(self.tab_3), _translate("MainWindow", "Data Management"))
        self.save_process_pb.setText(_translate("MainWindow", "Save Process"))
        self.process_data_pb.setText(_translate("MainWindow", "Process Data"))
        self.result_blue_label.setText(_translate("MainWindow", "Result (x):"))
        self.result_green_label.setText(_translate("MainWindow", "Result (x):"))
        self.result_red_label.setText(_translate("MainWindow", "Result (x):"))
        self.merge_spectra_cb.setText(_translate("MainWindow", "Merge spectra around ROI"))
        self.label_8.setText(_translate("MainWindow", "Enter comments concerning analysis:"))
        self.comments_edit.setToolTip(_translate("MainWindow", "Enter comments to be saved with analysis"))
        self.label_7.setText(_translate("MainWindow", "Formula\n"
"(the variable is \"x\"):"))
        self.label_20.setText(_translate("MainWindow", "<html><head/><body><p align=\"center\">Operation: (ex: a*x+b)</p><p align=\"center\">(ex: x if x&gt;0 else max(x))</p></body></html>"))
        self.measure_subtype_combo.setItemText(0, _translate("MainWindow", "dx"))
        self.measure_subtype_combo.setItemText(1, _translate("MainWindow", "amplitude"))
        self.measure_subtype_combo.setItemText(2, _translate("MainWindow", "x0"))
        self.label_9.setText(_translate("MainWindow", "Measurement Type: "))
        self.label_11.setText(_translate("MainWindow", "Measurement SubType: "))
        self.measure_type_combo.setItemText(0, _translate("MainWindow", "Cursor Intensity Integration"))
        self.measure_type_combo.setItemText(1, _translate("MainWindow", "Max"))
        self.measure_type_combo.setItemText(2, _translate("MainWindow", "Min"))
        self.measure_type_combo.setItemText(3, _translate("MainWindow", "Gaussian Fit"))
        self.measure_type_combo.setItemText(4, _translate("MainWindow", "Lorentzian Fit"))
        self.measure_type_combo.setItemText(5, _translate("MainWindow", "Exponential Decay Fit"))
        self.label_2.setText(_translate("MainWindow", "Formula parameters:"))
        self.do_operation_cb.setText(_translate("MainWindow", "Do operation"))
        self.toolBox_Analysis.setItemText(self.toolBox_Analysis.indexOf(self.page_measurement), _translate("MainWindow", "Measurement parameters"))
        self.apply_bkg_check.setText(_translate("MainWindow", "Apply Bkg"))
        self.set_whole_as_bkg_cb.setText(_translate("MainWindow", "Set whole carto as Bkg"))
        self.bkg_type.setItemText(0, _translate("MainWindow", "Sub."))
        self.bkg_type.setItemText(1, _translate("MainWindow", "Ratio"))
        self.do_zero_cb.setText(_translate("MainWindow", "Zero for negative values"))
        self.set_as_bkg_pb.setText(_translate("MainWindow", "Set as Bkg"))
        self.toolBox_Analysis.setItemText(self.toolBox_Analysis.indexOf(self.page_bkg), _translate("MainWindow", "Background settings"))
        self.show_despiked_cb.setText(_translate("MainWindow", "Show curve"))
        self.label_12.setText(_translate("MainWindow", "Threshold:"))
        self.remove_spikes_cb.setText(_translate("MainWindow", "Remove spikes"))
        self.label_16.setText(_translate("MainWindow", "Spikes width (pxls):"))
        self.label_17.setText(_translate("MainWindow", "Average width (pxls):"))
        self.label_18.setText(_translate("MainWindow", "Nloop:"))
        self.toolBox_Analysis.setItemText(self.toolBox_Analysis.indexOf(self.page_spikes), _translate("MainWindow", "Remove Spikes"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("MainWindow", "Analysis"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_TreeProperties), _translate("MainWindow", "Tree Properties"))
        self.label_15.setText(_translate("MainWindow", "Title"))
        self.label_14.setText(_translate("MainWindow", "Ylabel"))
        self.label_10.setText(_translate("MainWindow", "Aspect:"))
        self.aspect_combo.setItemText(0, _translate("MainWindow", "auto"))
        self.aspect_combo.setItemText(1, _translate("MainWindow", "equal"))
        self.plot_2D_pb.setText(_translate("MainWindow", "Plot 2D graph"))
        self.plot_1D_pb.setText(_translate("MainWindow", "Plot 1D graph"))
        self.label.setText(_translate("MainWindow", "Cmap:"))
        self.colorbar_cb.setText(_translate("MainWindow", "Colorbar"))
        self.update_plot_pb.setText(_translate("MainWindow", "Update"))
        self.save_plotting_pb.setText(_translate("MainWindow", "Save as"))
        self.label_13.setText(_translate("MainWindow", "Xlabel:"))
        self.use_labels_cb.setText(_translate("MainWindow", "Use Labels"))
        self.xlabel_edit.setText(_translate("MainWindow", "Xaxis (µm)"))
        self.ylabel_edit.setText(_translate("MainWindow", "Yaxis (µm)"))
        self.title_edit.setText(_translate("MainWindow", "ex: Photoluminescence map"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_plotting), _translate("MainWindow", "Plotting"))
        self.label_5.setText(_translate("MainWindow", "Current Group:"))
        self.label_4.setText(_translate("MainWindow", "Current Carto:"))
        self.label_6.setText(_translate("MainWindow", "Current Channel:"))
        self.label_3.setText(_translate("MainWindow", "Loaded Path:"))
        self.Filenamepath.setPlainText(_translate("MainWindow", "D:\\Data\\2016\\20161129"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.page), _translate("MainWindow", "Data infos"))
        self.menuFile.setTitle(_translate("MainWindow", "File"))
        self.menuLoad_Cartography_Labview.setTitle(_translate("MainWindow", "Load Cartography"))
        self.menuAdd_Cartography.setTitle(_translate("MainWindow", "Add Cartography"))
        self.menuExport.setTitle(_translate("MainWindow", "Export"))
        self.actionLoad_TDMS.setText(_translate("MainWindow", "Load TDMS"))
        self.actionLoad_Cartography_Labspec.setText(_translate("MainWindow", "Load Cartography Labspec"))
        self.actionLoad_Cartography_separated_files.setText(_translate("MainWindow", "Load Cartography separated files "))
        self.actionLoad_1D_Trace.setText(_translate("MainWindow", "Load 1D Trace"))
        self.actionLoad_Cartography_AFM.setText(_translate("MainWindow", "Load Cartography AFM"))
        self.actionDelete_Tree_Element.setText(_translate("MainWindow", "Delete Tree Element"))
        self.actionLabview.setText(_translate("MainWindow", "Labview"))
        self.actionSeparated_Files.setText(_translate("MainWindow", "Separated Files"))
        self.actionAFM.setText(_translate("MainWindow", "AFM"))
        self.actionLabview_add.setText(_translate("MainWindow", "Labview"))
        self.actionLabspec_add.setText(_translate("MainWindow", "Labspec"))
        self.actionSeparated_Files_add.setText(_translate("MainWindow", "Separated Files"))
        self.actionAFM_add.setText(_translate("MainWindow", "AFM"))
        self.action2D_image_as_txt.setText(_translate("MainWindow", "2D image as *.txt"))
        self.actionQuit.setText(_translate("MainWindow", "Quit"))
        self.actionLabspec_dual_mode.setText(_translate("MainWindow", "Labspec dual mode"))
        self.actionLabspec_Dual_Mode.setText(_translate("MainWindow", "Labspec Dual Mode"))
        self.actionLabspec.setText(_translate("MainWindow", "Labspec"))
        self.actionLabspec_Dual_Mode_add.setText(_translate("MainWindow", "Labspec Dual Mode"))

from pyqtgraph import PlotWidget
import QtDesigner_ressources_rc

if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    MainWindow = QtWidgets.QMainWindow()
    ui = Ui_MainWindow()
    ui.setupUi(MainWindow)
    MainWindow.show()
    sys.exit(app.exec_())

